import typer
from alembic.command import (
    downgrade,
    upgrade,
)
from alembic.config import Config

from popug_sdk.conf import settings


def migrate(
    revision: str = typer.Option("head"), down: bool = typer.Option(False)
) -> None:
    alembic_config = Config(settings.alembic.config)
    method = downgrade if down else upgrade
    method(alembic_config, revision)



from os import listdir
from os.path import (
    isfile,
    join,
)

import typer
from alembic.command import revision
from alembic.config import Config

from popug_sdk.conf import settings


def _get_revision_id(directory: str) -> str:
    versions_dir = join(directory, "versions")
    max_revision = 0
    for file in listdir(versions_dir):
        if isfile(join(versions_dir, file)):
            file_revision, *_ = file.split("_", 1)
            if file_revision.isdigit() and int(file_revision) > max_revision:
                max_revision = int(file_revision)
    return str(max_revision).rjust(4, "0")


def _get_next_revision_id(directory: str) -> str:
    revision_id = _get_revision_id(directory)
    return str(int(revision_id) + 1).rjust(4, "0")


def makemigrations(message: str = typer.Option("auto")) -> None:
    revision(
        Config(settings.alembic.config),
        message=message,
        autogenerate=True,
        rev_id=_get_next_revision_id(settings.alembic.directory),
    )



import os
import subprocess

from popug_sdk.conf import settings


def dbshell() -> None:
    os.environ["PGPASSWORD"] = settings.database.password
    subprocess.call(
        [
            "psql",
            "-h",
            settings.database.host,
            "-p",
            str(settings.database.port),
            "-U",
            settings.database.user,
            "-d",
            settings.database.database_name,
        ]
    )



from pydantic import BaseModel

from popug_sdk.conf.constants import (
    LOCALHOST,
    PortType,
)


class RedisSettings(BaseModel):
    host: str = LOCALHOST
    port: PortType = 6379
    db: int = 0



from typing import Any

from sqlalchemy.ext.declarative import (
    as_declarative,
    declared_attr,
)

__all__ = ("BaseModel",)


@as_declarative()
class BaseModel:
    id: Any
    __name__: str

    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



from typing import Iterable

from redis import ConnectionPool

from popug_sdk.conf.redis import RedisSettings

_redis_pools: dict[str, ConnectionPool] = {}


def init_redis_pool(config: dict[str, RedisSettings]) -> None:
    close_redis_pool(config.keys())
    _redis_pools.clear()

    for config_name, redis_pool_config in config.items():
        _redis_pools[config_name] = ConnectionPool(
            host=redis_pool_config.host,
            port=redis_pool_config.port,
            db=redis_pool_config.db,
        )


def close_redis_pool(config_names: Iterable[str]) -> None:
    for config_name in config_names:
        pool = _redis_pools.get(config_name)
        if pool:
            pool.disconnect()


def get_redis_pool(config_name: str = "default") -> ConnectionPool:
    if not _redis_pools:
        raise Exception("Call create_redis_pool() first")

    return _redis_pools[config_name]
